package com.redhat.ejb;

import javax.annotation.Resource;
import javax.ejb.EJBException;

import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.transaction.UserTransaction;

import javax.jms.*;


@Stateless
@TransactionAttribute(TransactionAttributeType.REQUIRED)
public class TransactionEJB {

	@PersistenceContext(unitName = "primary")
	private EntityManager em;

	@Resource(mappedName = "java:/JmsXA")
	ConnectionFactory connFactory;

	@Resource(mappedName = "java:jboss/jms/queue/exampleQueue")
	private Queue queueExample;

	@Resource(mappedName = "java:/JmsXA")
	private ConnectionFactory cf;

	@Resource
	private UserTransaction ut;
	
	private Connection connection;
	private MessageProducer publisher;
	private Session session;

	public void doTransaction() throws Exception {

		SimplePOJO p = new SimplePOJO();
		p.setKey(new java.util.Date().toString());
		p.setValue(new java.util.Date().toString());

		/* 2PC
		  Save a record on a DB and send a JMS message using XA Connection factory
		  The patched org/hornetq/ra/HornetQRAXAResource will raise XAException in the prepare() phase:
		  
		      XAException xaException = new XAException(XAException.XA_RBROLLBACK);
	          xaException.initCause(new Throwable("JMS Prepare failed in callee"));
	          throw xaException;
		 */
		
		try {
			em.persist(p);
			sendMessage("Hello");
		} catch (Exception e) {
		 
			// Not reached in CMT. Exception is re-thrown to the client automatically
			e.printStackTrace(); 
		    throw new EJBException(e);
		}
		 
	}

	 
	public void sendMessage(String txt) throws Exception {

		try {

			connection = cf.createConnection();
			session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);

			publisher = session.createProducer(queueExample);

			connection.start();

			TextMessage message = session.createTextMessage(txt);
			publisher.send(message);

		} catch (Throwable exc) {
			exc.printStackTrace();
			
			throw new Exception(exc);
		} finally {

			if (publisher != null)
				try {
					publisher.close();
				} catch (Exception ignore) {
				}
			if (session != null)
				try {
					session.close();
				} catch (Exception ignore) {
				}
			if (connection != null)
				try {
					connection.close();
				} catch (Exception ignore) {
				}

		}
	}

}
